Package: qqconf
Type: Package
Title: Creates Simultaneous Testing Bands for QQ-Plots
Version: 1.3.2
Authors@R: c(
    person("Eric", "Weine", email = "ericweine15@gmail.com", role = c("aut", "cre")),
    person("Mary Sara", "McPeek", email = "mcpeek@uchicago.edu", role = "aut"),
    person("Abney", "Mark", email = "abney@uchicago.edu", role = "aut"))
Description: Provides functionality for creating Quantile-Quantile (QQ) and Probability-Probability (PP) plots with simultaneous 
    testing bands to asses significance of sample deviation from a reference distribution <doi:10.18637/jss.v106.i10>.
License: GPL-3
Depends: R (>= 4.0.0)
SystemRequirements: fftw3 (>= 3.1.2)
Encoding: UTF-8
RoxygenNote: 7.2.3
Imports: MASS (>= 7.3-50), robustbase (>= 0.93-4), Rcpp
Suggests: knitr, rmarkdown, distr (>= 2.8.0)
Collate: 'one_sided.R' 'ppplot.R' 'qqconf-package.R' 'qqplot.R'
        'RcppExports.R' 'two_sided.R' 'utils.R'
VignetteBuilder: knitr
LinkingTo: Rcpp
URL: https://github.com/eweine/qqconf
BugReports: https://github.com/eweine/qqconf/issues
NeedsCompilation: yes
Packaged: 2023-04-14 19:38:46 UTC; ericweine
Author: Eric Weine [aut, cre],
  Mary Sara McPeek [aut],
  Abney Mark [aut]
Maintainer: Eric Weine <ericweine15@gmail.com>
Repository: CRAN
Date/Publication: 2023-04-14 23:00:21 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:41:44 UTC; unix
Archs: qqconf.so.dSYM
