# Generated by roxygen2: do not edit by hand

export(get_bounds_one_sided)
export(get_bounds_two_sided)
export(get_level_from_bounds_one_sided)
export(get_level_from_bounds_two_sided)
export(get_qq_band)
export(pp_conf_plot)
export(qq_conf_plot)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,qbeta)
importFrom(stats,qnorm)
importFrom(stats,sd)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(qqconf)
useDynLib(qqconf, .registration = TRUE)
