# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fft_get_level_from_bounds_two_sided <- function(b, B) {
    .Call(`_qqconf_fft_get_level_from_bounds_two_sided`, b, B)
}

