% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/one_sided.R
\name{get_bounds_one_sided}
\alias{get_bounds_one_sided}
\title{Calculates Rejection Region of One-Sided Equal Local Levels Test}
\usage{
get_bounds_one_sided(alpha, n, tol = 1e-08, max_it = 100)
}
\arguments{
\item{alpha}{Desired global significance level of the test.}

\item{n}{Size of the dataset.}

\item{tol}{(Optional) Relative tolerance of the \code{alpha} level of the
simultaneous test. Defaults to 1e-8.}

\item{max_it}{(Optional) Maximum number of iterations of Binary Search Algorithm
used to find the bounds. Defaults to 100 which should be much larger than necessary
for a reasonable tolerance.}
}
\value{
A list with components
\itemize{
  \item bound - Numeric vector of length \code{n} containing the lower bounds of the acceptance regions for the test of each order statistic.
  \item x - Numeric vector of length \code{n} containing the expectation of each order statistic. These are the x-coordinates for the bounds if used in a qq-plot.
  The value is \code{c(1:n) / (n + 1)}.
  \item local_level - Significance level \eqn{\eta} of the local test on each individual order statistic. It is equal for all order
  statistics and will be less than \code{alpha} for all \code{n} > 1.
}
}
\description{
The context is that n i.i.d. observations are assumed to be drawn
from some distribution on the unit interval with c.d.f. F(x), and it is
desired to test the null hypothesis that F(x) = x for all x in (0,1),
referred to as the "global null hypothesis," against the alternative F(x) > x for at least one x in (0, 1).
An "equal local levels" test is used, in which each of the n order statistics is
tested for significant deviation from its null distribution by a one-sided test
with significance level \eqn{\eta}.  The global null hypothesis is rejected if at
least one of the order statistic tests is rejected at level eta, where eta is
chosen so that the significance level of the global test is alpha.
Given the size of the dataset n and the desired global significance level alpha,
this function calculates the local level eta and the acceptance/rejection regions for the test.
The result is a set of lower bounds, one for each order statistic.
If at least one order statistic falls below the corresponding bound,
the global test is rejected.
}
\examples{
get_bounds_one_sided(alpha = .05, n = 10, max_it = 50)


}
